require 'spec_helper'
require 'pry'

describe Leap::CLI do
  let(:cli) { Leap::CLI.new }

  describe 'setup' do
    it 'invokes clone, remotes, dependencies' do
      expect(cli).to receive(:invoke).exactly(:thrice)

      cli.setup('username/project')
    end
  end

  describe 'clone' do
    context 'when directory with same name already exists' do
      before do
        allow(Dir).to receive(:exists?).and_return(true)
      end

      it 'does not clone project' do
        expect(cli).to_not receive(:`)
      end

      it 'warns user' do
        expect(cli).to receive(:say)
          .exactly(:once)
          .with('Project appears to already exist. Skipping clone.', :green)

        cli.clone('username/project')
      end
    end

    context 'when directory with same name does not exist' do
      before do
        allow(Dir).to receive(:exists?).and_return(false)
      end

      it 'clones project' do
        expect(cli).to receive(:`).with("git clone git@github.com:username/project.git")

        cli.clone('username/project')
      end

      it 'notifies user' do
        expect(cli).to receive(:say)
          .exactly(:once)
          .with("Cloning project.", :green)

        cli.clone('username/project')
      end
    end
  end

  describe 'dependencies' do
    context 'when brew is not present' do
      it 'installs brew' do
        allow_any_instance_of(Brew).to receive(:present?).and_return(false)

        expect_any_instance_of(Brew).to receive(:install_brew).exactly(:once)

        cli.dependencies
      end
    end

    context 'when brew is present' do
      it 'does not install brew' do
        allow_any_instance_of(Brew).to receive(:present?).and_return(true)

        expect_any_instance_of(Brew).to_not receive(:install_brew)

        cli.dependencies
      end
    end

    it 'installs dependencies' do
      expect_any_instance_of(Brew).to receive(:install_dependencies)

      cli.dependencies
    end
  end

  describe 'start' do
    context 'when start script exists' do
      before do
        allow(File).to receive(:exists?).and_return(false)
      end

      it 'alerts user of no start script' do
        expect(cli)
          .to receive(:say)
          .with('No start script', :red)
          .exactly(:once)

        cli.start
      end
    end

    context 'when start script does not exist' do
      before do
        allow(File).to receive(:exists?).and_return(true)
        allow_any_instance_of(Object).to receive(:exec).and_return(double)
      end

      it 'executes that script' do
        expect_any_instance_of(Object)
          .to receive(:exec)
          .with('.leap/start')
          .exactly(:once)

        cli.start
      end
    end
  end
end
